#######################################################################
#
# NCM component for uicmnconfig daemon
#
# Charles Loomis <charles.loomis@cern.ch>
# Version 2.0.4 : 24/03/05 13:17
#
# ** Generated file : do not edit **
#
#######################################################################

package NCM::Component::uicmnconfig;

use strict;
use NCM::Component;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;
use File::Copy;
use File::Path;

use EDG::WP4::CCM::Element;


local(*DTA);


##########################################################################
sub Configure($$@) {
##########################################################################
    
    my ($self, $config) = @_;

    # Define paths for convenience. 
    my $base = "/software/components/uicmnconfig";
    my $vobase = "/system/vo";
    my $edgpath = "/system/edg/config/EDG_LOCATION";

    # Default location for EDG software.
    my $edgloc = "/opt/edg";
    if ($config->elementExists($edgpath)) {
        $edgloc = $config->getValue($edgpath);
    }

    # Base for generating VO-specific configuration files. 
    my $basedir = "$edgloc/etc";
    if ($config->elementExists("$base/basedir")) {
        $basedir = $config->getValue("$base/basedir") . "/etc";
    }

    # If the list of VO information exists, process each entry. 
    if ($config->elementExists("$vobase")) {

        # Get the list of VOs to configure, should be a list.
        my $volist = $config->getElement("$vobase");
        while ($volist->hasNextElement()) {

            my $element = $volist->getNextElement();
            
            my $index = $element->getName();

            # Get a hash of this and iterate over the entries.
            my $voname = 
                $config->getElement("$vobase/$index/name")->getValue();
            my %entry = 
                $config->getElement("$vobase/$index/services")->getHash();

            # Extract the required information. 
            my $myproxy = $entry{myproxy}->getValue();
            my @nshosts = $entry{nshosts}->getList();
            my @lbhosts = $entry{lbhosts}->getList();

            # Optional information.
            my $hlr = undef;
            my $property = $entry{hlr};
            $hlr = $property->getValue() if ($property);

            my $contents = process_entry($voname, $myproxy, $hlr, 
                                         \@nshosts, \@lbhosts);

            # Will return undefined value on an error. 
            if (!defined($contents)) {
                $self->error("error creating uicmnconfig file for $voname");
                next;
            }

            # Ensure that the necessary directory exists.
            mkpath("$basedir/$voname/",0,0755);

            # Now just create the new configuration file.  Be careful to save
            # a backup of the previous file if necessary. 
            my $fname = "$basedir/$voname/edg_wl_ui.conf";
            
            if ( ! -e $fname ) {
                
                # Configuration file doesn't exist yet.  Create it. 
                open ( CONF,">$fname" );
                print CONF $contents;
                close (CONF);
                $self->log("$fname created");
                
            } else {
                
                # Already exists. Make backup and create new file. 
                my $result = LC::Check::file( $fname,
                                              backup => ".old",
                                              contents => $contents,
                                              );
                $self->log("$fname updated") if $result;
            }

        }
    }

    return 1;
}

# Generate the contents for a single VO entry.
sub process_entry {

    my ($voname, $myproxy, $hlr, $nsref, $lbref) = @_;

    my @nshosts = @$nsref;
    my @lbhosts = @$lbref;

    my $contents = "#\n# Generated by ncm-uicmnconfig on " . 
        localtime() . "\n#\n";
    $contents .= "[\n";
    $contents .= "  VirtualOrganisation = \"$voname\";\n";
    $contents .= "  MyProxyServer = \"$myproxy\";\n";
    $contents .= "  HRLLocation = \"$hlr\";\n" if defined($hlr);

    # Add the LB addresses.
    # as of LCG2.3 we need to repeat the LBaddresses stanza for
    # each NSaddresses entry. This does not provide the full functionality
    # of the NS/LB pairing, but at least makes it rubust.
    # every set of LB servers is now matched up with each of the NS-es
    $contents .= "  LBAddresses = {\n";
    my $stanzafirst = 1;
    my $first;
    for my $istanza ( 0 .. $#nshosts ) {
      $contents .= ",\n" unless($stanzafirst);
      $contents .= "                 {\n";
      $first = 1;
      foreach my $element (@lbhosts) {
        my $v = $element->getValue();
        $contents .= ",\n" unless($first);
        $contents .= '                   "' . $v . '"';
        $first = 0;
      }
      $contents .= "\n                 }";
      $stanzafirst = 0;
    }
    $contents .= "\n                };\n";

    # Add the NS addresses.
    $contents .= "  NSAddresses = {\n";
    $first = 1;
    foreach my $element (@nshosts) {
        my $v = $element->getValue();
        $contents .= ",\n" unless($first);
        $contents .= '                 "' . $v . '"';
        $first = 0;
    }
    $contents .= "\n                };\n";

    # Finish off the file.
    $contents .= "]\n";

    return $contents;
}

1;      # Required for PERL modules
